/**
 * 与业务相关的公共代码都写在这里
 * zhaohuihua, 2016-10-29
 */

// 拦截全局错误
+function(window, Config) {
	window.onerror = function(msg, url, line) {
		if (Config.hybrid && Config.debug) {
			alert("Error: " + msg + "\nURL: " + url + "\nLine: " + line);
		}
		return false; // 返回值为false则仍然输出到控制台
	};
}(window, window.Config || {});

// Utils工具类
+function($) {
	
	var showLoading = function(){
		$.zloading("show");
	}
	
	var stopLoading = function(){
		$.zloading("hide");
	}

	/** 判断返回结果是否成功 **/
	var succ = function(json) {
		if(json.code == "UNAUTHORIZED"){
			Utils.nav.next("/views/home/login.html");
		}
		return json && json.code == "SUCCESS";
	};

	var toast = function(message) {
		window.top.mui.toast(message);
	};

	var concatUrl = function(base, uri) {
		if (!base.endsWith('/')) { base += '/'; }
		if (uri == undefined) {
			return base;
		} else if (/^https?:/.test(uri) || uri.charAt(0) == '/') {
			return uri;
		} else {
			return base + uri;
		}
	};
	var getBaseUrl = function(href) {
		if (/^https?:/.test(href) || href.charAt(0) == '/') {
			return href;
		} else {
			// var base = $("meta[name=base-url]").attr("content");
			var base = Config.baseurl;
			return base + (href ? (href.startsWith('/') ? href.substring(1) : href) : "");
		}
	};
	
	// 将绝对路径转换为BaseUrl
	// toBaseUrl("http://xxx.com/app/user/index.html"); --> /app/user/index.html
	var toBaseUrl = function(url) {
		if (!url) { url = window.location.href; }
		return url.replace(/[\?\#].*/, "").replace(/.*\/?views/, "");
	};

	var setCacheItem = function(key, object) {
		if (object == undefined) {
			plus.storage.removeItem(key);
		} else {
			// 如果不是对象, 则保存为{$:xxx}, 以便能正确还原数据类型
			// TODO Date类型还不能正确还原
			if (!$.isPlainObject(object) && !$.isArray(object)) {
				object = {$:object};
			}
			plus.storage.setItem(key, JSON.stringify(object));
		}
	};
	var getCacheItem = function(key, parse) {
		var string = plus.storage.getItem(key);
		if (!string) { return null; } 
		if (parse === false) {
			return string;
		} else {
			if (string.startsWith('[')) {
				return JSON.parse(string);
			} else if (string.startsWith('{')) {
				// 如果是{$:xxx}格式, 则取json.$以便还原数据类型
				var json = JSON.parse(string);
				if ("$" in json) {
					var count = 0;
					for (var k in json) { count ++; }
					if (count == 1) { return json.$; }
					else { return json; }
				} else {
					return json;
				}
			} else {
				return string;
			}
		}
	};
	var delCacheItem = function(key, parse) {
		var old = getCacheItem(key, parse);
		plus.storage.removeItem(key);
		return old;
	};

	var openWindow = function(replace, url, hashParams, passParams) {
		if (passParams) {
			// 参数发送至子页面
			sendPassParams(url, passParams);
		}
		var options = {};
		if (typeof(url) == "string") {
			options.url = url;
		} else {
			options = url;
		}
		if (!options.id) {
			var beginIndex = options.url.lastIndexOf('views/')+6;
			var index = options.url.lastIndexOf('.');
			if (index > 0) {
				options.id = options.url.substring(beginIndex, index);
			}else{
				options.id = options.url.substring(beginIndex , options.url.length);
			}
		}
		if (hashParams) {
			options.url += "#" + $.param(hashParams);
		}
		var currentWebview;
		if (replace) {
			currentWebview = plus.webview.currentWebview();
			var parentWebview = currentWebview.parent();
			if(parentWebview){
				currentWebview = parentWebview;
			}
			setTimeout(function(){
				plus.webview.close(currentWebview, "none", 0);
			}, 500);
		}
		mui.openWindow(options);
	};
	// 页面间传递参数
	var sendPassParams = function(url, json) {
		var passid = "PassParams:" + toBaseUrl(url);
		setCacheItem(passid, json);
	};
	// 接收传递的参数
	var getPassParams = function(warnOnUndefined) {
		if (!window.PassParams) {
			var url = toBaseUrl();
			var passid = "PassParams:" + url;
			var data = delCacheItem(passid);
			if (warnOnUndefined !== false && data == undefined) {
				$.log.warn("PassParams not found: " + url);
			}
			window.PassParams = data || {};
		}
		return window.PassParams;
	};
	if (window != window.top) {
		getPassParams(false); // 立即接收参数, 只有子页面执行
	}

	if (!window.Utils) { window.Utils = {}; }
	var ua = window.navigator.userAgent;
	$.extend(window.Utils, {
		showLoading: showLoading,
		stopLoading: stopLoading,
		// https://github.com/faisalman/ua-parser-js/blob/master/src/ua-parser.js
		ios: !!/(ip[honead]+)(?:.*os\s([\w]+)*\slike\smac|;\sopera)/i.exec(ua),
		android: !!/android/i.exec(ua),
		// 事件
		events: window.top.$.zhh.events,
		/**
		 * 显示消息提示框
		 * @param message 消息提示文本
		 */
		toast: toast,
		/**
		 * @description 判断返回结果是否成功
		 * @example
		 * Utils.succ(json); // json = { resultCode:'100', resultJson:{} }
		 */
		succ: succ,
		/**
		 * @description 获取接口地址
		 * @example
		 * Utils.getBaseUrl('zhrws/ZHRWS2117');
		 */
		getBaseUrl:getBaseUrl,
		/**
		 * 如果字段为空，接口会返回{}，此时需要改为空字符串
		 */
		getString: function(string) {
			if (string == undefined) { return ""; }
			if (string == "[object Object]" || string == "undefined") { return ""; }
			if ($.isPlainObject(string)) { return ""; }
			return string.toString();
		},
		/**
		 * @description 获取数组
		 * @example
		 * Utils.getArray(undefined);  =  []
		 * Utils.getArray({});  =  []
		 * Utils.getArray({xx:value});  =  [{xx:value}]
		 */
		getArray:function(array) {
			if (array == undefined) { return []; }
			if ($.isArray(array)) { return array; }
			if ($.isPlainObject(array) && $.isEmptyObject(array)) { return []; }
			return [array];
		},
		/**
		 * 滚动至指定位置
		 * Utils.muiScrollTo(0);
		 * Utils.muiScrollTo("top");
		 * Utils.muiScrollTo("bottom");
		 * Utils.muiScrollTo($elem);
		 */
		muiScrollTo: function(target, time, easing) {
			var api = mui(".mui-scroll-wrapper").scroll();
			if (!api.reLayout || !api.scrollTo) { return; } // 没有滚动条
			api.reLayout();
			var y;
			if ($.dt.isNumeric(target)) {
				y = target * 1;
			} else if (target == "top") {
				y = 0;
			} else if (target == "bottom") {
				y = api.maxScrollY;
			} else if ($.dt.isElement(target)) {
				y = api.y - $(target).offset().top + $("header").height() + 10;
			} else {
				return;
			}
			var time = time || api.options.scrollTime;
			api.scrollTo(api.x, y, time, easing);
		},
		/**
		 * 向下一页面传递参数
		 * Utils.sendPassParams(json);
		 */
		sendPassParams:sendPassParams,
		/**
		 * 接收上一页面传递的参数
		 * Utils.getPassParams();
		 * @return json
		 */
		getPassParams:getPassParams,
		nav: {
			/**
			 * 返回
			 * Utils.nav.back(); // 返回上一页
			 * Utils.nav.back('home/login'); // 
			 * Utils.nav.back('homePage'); // 返回首页
			 */
			back: function(page) {
				if(page){
					if(page == "homePage"){
						page = plus.webview.getLaunchWebview();
					}
					plus.webview.show(page);
				}else{
					mui.back(page); // 不用判断xview了, 如果是xview模式, mui.back已经被重新定义了
				}
			},
			/**
			 * 重新加载
			 * Utils.nav.reload()
			 */
			reload: function() {
				if (Config.xview) {
					var iframe = Config.xview.subwindow();
					iframe && iframe.location.reload();
				} else {
					window.top.location.reload();
				}
			},
			/**
			 * 跳转至下一页
			 * Utils.nav.next(url:string, hashParams:json, passParams:json);
			 */
			next: function(url, hashParams, passParams) {
				return openWindow(false, url, hashParams, passParams);
			},
			/**
			 * 替换当前页
			 * Utils.nav.replace(url:string, hashParams:json, passParams:json);
			 */
			replace: function(url, hashParams, passParams) {
				return openWindow(true, url, hashParams, passParams);
			},
			/**
			 * 关闭指定页面
			 * Utils.nav.close(id);//根据id关闭
			 * Utils.nav.close(WebviewObject);//根据webview对象关闭
			 */
			close: function(object) {
				plus.webview.close(object, "none", 0);
			},
			/**
			 * 获取缓存的页面
			 * var iframe = Utils.nav.view('list.html');
			 * var xxx = $(iframe.document.body).find("xxx");
			 * Utils.nav.view(-n); == 上n个页面
			 * Utils.nav.view(+n); == 第n个页面
			 */
			view: function(name) {
				if (Config.xview) {
					return Config.xview.subwindow.apply(Config.xview, arguments);
				}
			},
			/**
			 * 执行缓存页面的方法
			 * Utils.nav.invoke('home/offcanvas', method, data);//执行id页面方法
			 * Utils.nav.invoke(WebviewObject, method, data);//执行WebviewObject页面方法
			 */
			invoke: function(page, method, data) {
//				if (Config.xview) {
//					return Config.xview.invoke.apply(Config.xview, arguments);
//				}
				var webview;
				if(typeof(page) == "string"){
					if(page == "homePage"){
						webview = plus.webview.getLaunchWebview();
					}else{
						webview = plus.webview.getWebviewById(page);
					}
				}else{
					webview = page;
				}
				if(webview){
					if(typeof(data) != "string"){
						data = JSON.stringify(data);
					}
					webview.evalJS(method+"('"+data+"')");
				}
			}
		},
		cache: {
			/**
			 * Utils.cache.set(key:string, object:string|json);
			 * @description 将对象保存至缓存
			 * @example
			 * Utils.cache.set('USER_INFO', { id:10001, name:'xxx' });
			 */
			set:setCacheItem, 
			/**
			 * Utils.cache.get(key:string)
			 * Utils.cache.get(key:string, parse:true|false)
			 * @description 从缓存中读取对象
			 * @param parse 是否解析为对象, 默认为true
			 * @example
			 * Utils.cache.get('USER_INFO'); = { id:10001, name:'xxx' }
			 */
			get:getCacheItem,
			/**
			 * Utils.cache.del(key:string);
			 * Utils.cache.del(key:string, parse:true|false);
			 * @description 从缓存中删除对象
			 * @param parse 是否解析为对象, 默认为true
			 * @return 返回被删除的对象
			 * @example
			 * Utils.cache.del('USER_INFO');
			 */
			del:delCacheItem
		}
	});

}(jQuery);

+function($) {
	// 清除输入框内容, 触发change事件
	$(document).on('tap', '.mui-icon-clear', function(){
		var me = $(this);
		me.prevAll('input').change();
	});
}(jQuery);


/**
 * -------------------------------------------------------------
 * 照片附件插件
 * -------------------------------------------------------------
	<div class="ln uploads do-camera" data-options="limitCount:5, imageName:'imageInfos[i].imagePath'">
		<a class="camera"></a>
	</div>
	$(".do-camera").on("change", function(e, json) {
		// json = { total:number, url:string, index:integer } // 新增时有url, 删除时有index
	});
 * -------------------------------------------------------------
	<div class="ln uploads edit-camera" data-options="imageName:'imageInfos[i].imagePath'">
		<a class="camera"></a>
	</div>
	$(".edit-camera").zcamera({
		on: {
			// json = { url:string }
			add: function(api, json) { if (...) { api.add(json); } },
			// json = { index:integer }
			del: function(api, json) { if (...) { api.del(json); } }
		}
	});
 * -------------------------------------------------------------
 */
+function($) {
	"use strict"

	var API_KEY = 'zloading';
	$.fn.zcamera = function(options) {
		if (options === 'api') {
			return init.call(this[0]);
		} else if (typeof(options) == 'string' && Camera.prototype[options]) {
			var api = init.call(this[0]);
			return api[options].apply(api, Array.prototype.splice.call(arguments, 1));
		}

		return this.each(function() {
			init.call(this, options);
		});
	};
	
	var init = function(options) {
		var me = $(this);
		var api = me.data(API_KEY);
		if (!api) {
			$(this).data(API_KEY, api = new Camera(this, options));
		}
		return api;
	};

	// 构造函数
	var Camera = function(element, options) {
		this.$element = $(element);
	    this.options = $.extend(true, { }, $.fn.zcamera.defaults, this.$element.zoptions(), options);
	    this.init();
	};
	// 初始化
	Camera.prototype.init = function() {
		var api = this;
		var me = api.$element;
		var o = api.options;
		if (me.find(o.selector.add).length == 0) {
			me.append(o.template.camera);
		}
		me.off("tap.zcamera");
		me.on("tap.zcamera", o.selector.add, function() {
			var total = me.find("a.image").length;
			var params = { limitCount:o.limitCount, currCount:total };
			Utils.app.camera({ title:o.title, width:o.width, height:o.height, callback:function(base64string) {
				o.on.add.call(this, api, { url:base64string });
			}});
		});
		me.on("tap.zcamera", o.selector.del, function() {
			var $image = $(this).closest(o.selector.image);
			var index = me.find(o.selector.image).index($image[0]);
			o.on.del.call(this, api, { index:index, $image:$image });
		});
	};
	Camera.prototype.clear = function() {
		var api = this;
		var me = api.$element;
		var o = api.options;
		me.find(o.selector.image).remove();
	};
	/**
	 * 增加图片
		json = { 
			url:string, // 图片地址, 填写在<img src="{url}">中
			originalUrl:string, // 原图片地址, 用于预览时查看的大图
			deletable:boolean, // 是否可删除, 默认可删除
			uploadable: boolean, // 是否可上传, 默认可上传
			extra:json // 其他附加属性, <a class="image" data-extra="{extra}">
		}
	 */ 
	Camera.prototype.add = function(json) {
		if ($.dt.isString(json)) { json = { url:json }; }
		var api = this;
		var me = api.$element;
		var o = api.options;
		var tpl = baidu.template(o.template.image);
		var html = tpl($.extend(true, { imageName:o.imageName, imageGroup:o.imageGroup }, json));
		var $image = $(html);
		me.append($image);
		var total = me.find("a.image").length;
		if (total >= o.limitCount) {
			me.find(o.selector.add).hide();
		}
		me.trigger("change", $.extend(true, { total:total, target:$image }, o, json));
		return $image;
	};
	Camera.prototype.del = function(json) { // json = { index:integer }
		var api = this;
		var me = api.$element;
		var o = api.options;
		var $image = me.find(o.selector.image).eq(json.index);
		$image = $image.remove();
		var total = me.find("a.image").length;
		if (total < o.limitCount) {
			me.find(o.selector.add).show();
		}
		me.trigger("change", $.extend(true, { total:total, target:$image }, o, json));
	};
	
	$.fn.zcamera.defaults = {
		title: '上传图片',
		width: "100",
		height: "100",
		limitCount: 5, // 总共允许选择几张照片
		imageName:"images", // 图片名称
		imageGroup:"images", // 预览图片的分组
		selector:{ image:".image", add:".camera", del:".delete" },
		template:{
			camera:'<a class="camera"></a>',
			image:
				'<a class="image" data-extra="<#:h=JSON.stringify(this.extra)#>">' + 
				'	<img src="<#=this.url#>" <#if(this.uploadable!=false){#>data-zupload="<#=this.imageName#>"<#}#> data-preview-src="<#=this.originalUrl#>" data-preview-group="<#=this.imageGroup#>" />'+
				'	<#if(this.deletable!=false){#><i class="delete"></i><#}#>' + 
				'</a>'
		},
		on: {
			// json = { url:string }
			add: function(api, json) { api.add(json); },
			// json = { index:integer }
			del: function(api, json) { api.del(json); }
		}
	};
}(jQuery);

+function($) {
	/*
	 * 自动登录
	 * options:{
	 	* ajaxParam:{account:"15272635262", password:"123456"}, //如果不设置则从缓存取
	 	* loading:true, //是否有加载动画，如果不设置则没有
	 	* succ:function(){}, //成功以后执行的代码
	 	* fail:function(){} //失败以后执行的代码
	 * }
	 */
	var loginCacheKey = "loginUserinfo";//用户信息缓存标识
	var loginUserinfoAjaxParamKey = "loginUserinfoAjaxParam";//用户登录ajaxParam
	var autoLogin = function(options){
		if(!options){
			options = {};
		}
		if(!options.ajaxParam){
			var loginUserinfoAjaxParam = Utils.cache.get(loginUserinfoAjaxParamKey);
			if(!loginUserinfoAjaxParam){
				return;
			}
			options.ajaxParam = loginUserinfoAjaxParam;
		}
  		options.ajaxParam.autologin = true;
		var url = Utils.getBaseUrl("user/login.json");
		$.zajax(url, options.ajaxParam, options.loading, {
			succ: function(json) {
				if(json.code == "SUCCESS"){
					var loginUserinfo = json.body;
					Utils.cache.set(loginUserinfoAjaxParamKey, options.ajaxParam);
					Utils.cache.set(loginCacheKey, loginUserinfo);
					options.succ && options.succ(json);
					console.log("自动登录成功！");
				}else{
					options.fail && options.fail(json);
					console.log("自动登录失败！");
				}
			},
			fail: function(json) {
				options.fail && options.fail(json);
				console.log("自动登录失败！");
			}
		});
	}
	
	//获取登录用户信息
	var getLoginUserinfo = function(){
		return Utils.cache.get(loginCacheKey);
	}
	
	//判断是否登录
	var checkLoginState = function(){
		var loginUserinfo = Utils.cache.get(loginCacheKey);
		if(loginUserinfo){
			return true;
		}else{
			return false;
		}
	}
	
	/*
	 * 退出登录
	 */
	var outLogin = function(){
		Utils.cache.del(loginCacheKey);
		Utils.cache.del(loginUserinfoAjaxParamKey);
		Utils.nav.back();
	}
	
	window.loginUtils = {
		autoLogin:autoLogin,//自动登录
		getLoginUserinfo:getLoginUserinfo,//获取登录用户信息
		checkLoginState:checkLoginState,//判断是否登录
		outLogin:outLogin//退出登录
	}
}(jQuery);


+function($) {
	/*
	 * 设置验证码秒数
	 * getCodeBox：按钮框
	 * secondBox：显示秒数的框
	 */
	var setCodeSecond = function(getCodeBox, secondBox){
		getCodeBox.addClass("disabled");
		var second = 60;
		var setCodeSecondIntervalFn = function(){
			secondBox.html("("+second+")");
			second--;
			if(second < 1){
				clearInterval(setCodeSecondInterval);
				getCodeBox.removeClass("disabled");
				secondBox.html("");
			}
		}
		setCodeSecondIntervalFn();
		var setCodeSecondInterval = setInterval(setCodeSecondIntervalFn, 1000);
	}
	
	window.thisUtils = {
		setCodeSecond:setCodeSecond//设置验证码倒计时
	}
}(jQuery);