/*!
 * 通用表单校验
 * 赵卉华 / zhaohuihua@126.com / 2012-09-05 / 2014-01-25 / 2015-03-02
 */
(function($) {
	"use strict";
	$.fn.zhhvalidate = function(callback) { // 校验
		return this.each(function(i) {
			var self = $(this);
			if(self.is("form")) {
				if (callback === "init") {
					init.call(this);
				} else if (callback === "clear") {
					$(this).find(".vld-line").each(function() {
						clear.call(this);
					});
				} else {
					vldform.call(this, callback);
				}
			} else if(self.is(".vld-line")) {
				if (callback === "clear") {
					clear.call(this);
				} else {
					vldline.call(this, callback);
				}
			}
		});
	};


	var bind = (function() {
		var map = {};
		var push = function(join) {
			var key = this;
			if (!map[key]) { map[key] = []; }
			var array = map[key];
			if (arguments.length == 0) {
				return array;
			} else if (typeof(join) == "boolean") {
				return join ? array.join(",") : array;
			} else {
				var selector = $.isArray(join) ? join : arguments;
				Array.prototype.push.apply(array, selector);
				return array;
			}
		};
		return {
			input:  function() { return push.apply("input",  arguments); },
			blur:   function() { return push.apply("blur",   arguments); },
			click:  function() { return push.apply("click",  arguments); },
			submit: function() { return push.apply("submit", arguments); }
		};
	})();

	var rules = (function() {
		var map = {};
		var exist = function(name) {
			return name && name in map;
		};
		// 获取规则
		var get = function(name) {
			return map[name];
		};
		// 新增或修改规则
		var put = function(name, fn) {
			if (!name) {
				throw "rule name is undefined.";
			} else if (inArray(name, CONFIG.keyword) >= 0) {
				throw "rule name " + name + " is keyword.";
			} else if (/^\./.test(name)) {
				throw "rule name " + name + " cannot begin with a dot.";
			} else if (!CONFIG.regexp.naming.test(name)) {
				throw "rule name " + name + " does not conform to naming conventions.";
			} else if (!$.isFunction(fn)) {
				throw "rule handler isn't a function.";
			}
			map[name] = fn;
		};
		// 删除规则
		var remove = function(name) {
			delete map[name];
		};
		return { exist:exist, get:get, put:put, remove:remove }
	})();


	// 初始化需要绑定事件的元素
	var input = [
  	    ".vld-line input[type=text]",
  	    ".vld-line input[type=password]",
  	    ".vld-line textarea"
  	];
	bind.input(input);
	bind.blur(input);
	bind.click(
	    ".vld-line select option",
	    ".vld-line input[type=radio]",
	    ".vld-line input[type=checkbox]"
	);
	bind.submit(
  	    "a.vld-submit",
  	    "button.vld-submit",
  	    "input[type=button].vld-submit"
  	);

	var init = function() {
		var form = $(this);
		// IE8修改任何属性都会触发propertychange, 例如addClass(), 有时会导致内存溢出
		// 因此需要记录原来的值与目前的值对比
		var LASTVALUE = "LAST-VALUE";
		form.find(bind.input(true)).each(function() {
			$(this).data(LASTVALUE, $(this).val());
		});
		var change = /msie/i.test(navigator.userAgent) ? "propertychange" : "input";
		form
		// 输入时校验
		.on(change, bind.input(true), function() {
			var value = $(this).val();
			var last = $(this).data(LASTVALUE);
			if (value == last) { return; }
			$(this).data(LASTVALUE, value);
			$(this).closest(".vld-line").zhhvalidate();
		})
		// 失去焦点时校验
		.on("blur", bind.blur(true), function() {
			var value = $(this).val();
			var last = $(this).data(LASTVALUE);
			if (value == last) { return; }
			$(this).closest(".vld-line").zhhvalidate();
		})
		// 点击时校验
		.on("click", bind.click(true), function() {
			$(this).closest(".vld-line").zhhvalidate();
		})
		// 点击提交按钮
		.on("click", bind.submit(true), function() {
			var $form = $(this).closest("form");
			$form.zhhvalidate(function(e) {
				if (e.passed) { $form.submit(); }
			});
			return false;
		});
	};


	/**
	 * 校验FORM表单
	 * callback回调参数说明:
	 * passed(boolean) FORM表单校验是否通过
	 * config(json) 配置项
	 * list(json[]) 每一个表单行的校验校验结果, json内容见vldline的回调参数说明
	 */
	var vldform = function(callback) {
		var passed = true, list = [], first;
		// 逐一校验LINE
		$(this).find(".vld-line").each(function() {
			vldline.call(this, function(e) {
				list.push(e);
				// 只要有一条校验失败, 整个表单就不通过
				if (!e.passed) {
					passed = false;
					if (!first) { first = e.target; }
				}
			});
		});
		if (!passed) { // 未全部通过, 设置焦点为第1个校验失败的目标
			if(first && first.is(":visible")) { // 设置焦点
				try { first.focus().select(); } catch(ignore) {}
			}
		}
		$.isFunction(callback) && callback.call(this, { passed:passed, list:list });
	};
	/**
	 * 校验表单行
	 * callback回调参数说明:
	 * passed(boolean)       表单行校验是否通过
	 * tips(jQuery Object)   导致校验失败规则
	 * target(jQuery Object) 校验目标, 即输入框
	 * value(String)         校验目标(输入框)的值
	 * config(json)          配置项
	 */
	var vldline = function(callback) {
		var self = this, $line = $(this), passed = true, event;

		// 隐藏掉所有的规则
		// false=不处理错误样式, 因为IE8的addClass()也会触发propertychange
		clear.call(this, true, false);
		// 找到所有的校验规则, 逐一执行校验
		$line.find("[class*='vld-']").each(function() {
			var classes = $(this).attr("class").split(/\s+/);
			for (var i = 0, len = classes.length; i < len; i++) {
				// 不需要检查系统关键字了, 增加的时候已经做了检查

				var type = classes[i].replace(/^vld\-/, "");

				var fn = CONFIG.rules.get(type);
				if (!$.isFunction(fn)) { continue; }

				var $tips = $(this); // 校验规则

				var e = new Event($line, $tips);
				// 未找到输入框, 认为成功
				// return true, 继续下一条规则, 有可能下一条的data-target值不同
				if(e.target.length == 0) { return true; }

				event = e;
				e.rule = { type:type, fn:fn };

				// 执行校验函数(调用前加上config参数)
				var result = fn.call(self, e);
				// 明确的return false, return json即为校验不通过
				// 没有return; return true; return; 都认为校验通过
				// 校验通过就将错误提示隐藏掉, 但强制显示的除外(如密码强度提示)
				if (result === true || result == undefined) { // 通过
					if (inArray(type, CONFIG.visible) < 0) { // 非强制显示
						e.tips.hide();
					}
				} else { // 不通过
					e.setTipsText(result);
					e.tips.show();
					// 校验不通过返回false, 就不会执行下一条规则了
					return passed = false;
				}
			}
			return passed;
		});
		if (event) {
			if (!passed) { // 校验未通过
				event.line.addClass("vld-error-line");
				event.target.addClass("vld-error-input");
			} else { // 校验通过
				if (event.value.length > 0) { // 输入框不为空, 显示成功图标
					event.tips = $line.find(".vld-passed").show();
				}
				event.line.removeClass("vld-error-line");
				event.target.removeClass("vld-error-input");
				delete event.tips;
			}
			event.passed = passed;
		} else {
			event = { passed:passed };
		}
		$.isFunction(callback) && callback.call(self, event);
	};

	// 初始化, 隐藏所有的规则, 显示默认的提示信息
	// msg=默认提示信息也隐藏, 默认不隐藏
	// err=错误样式也隐藏, 默认隐藏
	var clear = function(msg, err) {
		$(this).find(".vld-passed").hide()
		.end().find("[class*='vld-']").each(function() {
			var list = $(this).attr("class").split(/\s+/);
			for (var i = 0, len = list.length; i < len; i++) {
				var type = list[i].replace(/^vld\-/, "");
				if (CONFIG.rules.exist(type) && inArray(type, CONFIG.visible) < 0) {
					$(this).hide();
				}
			}
		});
		if (msg) { $(this).find(".vld-message").hide(); }
		else { $(this).find(".vld-message").show(); }
		if (err != false) {
			$(this).removeClass("vld-error-line")
			.end().find(".vld-error-input").removeClass("vld-error-input");
		}
	};

	var inArray = function(elem, array) {
		for (var i = 0, length = array.length; i < length; i++) {
			if (array[i].constructor == RegExp && typeof(elem) == "string") {
				if (array[i].test(elem)) {
					return i;
				}
			} else if (array[i] === elem) {
				return i;
			}
		}
		return -1;
	};

	var Event = function($line, $tips) {
		this.line = $line;
		this.tips = $tips;
		// 获取校验的目标, 即输入框
		this.target = this.findTarget();
		// 获取校验的目标, 即输入框
		this.label = this.findLabelText();
		// 获取校验的值, 即输入框的值
		this.value = this.getValue();
	};
	Event.prototype = {
		inArray:inArray,
		// 查找校验目标(即输入框)
		// 查找顺序:
		// 1. rule指明的data-target
		// 2. class="vld-target"
		// 3. input, textarea, select
		findTarget: function($line, $tips) {
			if (!$line) { $line = this.line; }
			if (!$tips) { $tips = this.tips; }
			var $target = [];
			var target = $tips.data("target");
			if (target) {
				$target = $line.find(target);
			}
			if ($target.length == 0) {
				$target = $line.find(".vld-target");
			}
			if ($target.length == 0) {
				$target = $line.find("input, textarea, select");
			}
			if($target.is("[type=checkbox],[type=radio]")) {
				return $target;
			} else {
				return $target.filter(":first");
			}
		},
		// 查找校验目标的提示文本
		// 查找顺序:
		// 2. target指明的data-label文本
		// 3. class="vld-label"标签中的文本
		// 4. label标签的文本
		findLabelText: function($line, $target) {
			if (!$line) { $line = this.line; }
			if (!$target) { $target = this.target; }
			var text = $target.data("label");
			if (!text) {
				text = $line.find(".vld-label, label").eq(0).text();
			}
			// 替换掉前面的*和后面的:
			// 如 * 登录用户名 : 替换为登录用户名
			return text && $.trim(text).replace(/(^\*\s*|\s*[:：]$)/g, "");
		},
		// 获取校验的值, 即输入框的值
		getValue: function($target) {
			if (!$target) { $target = this.target; }
			// 获取校验的值, 即输入框的值
			var value = $target.val();
			// 取值默认会截掉前后空格, 只有在声明vld-trim=false时才不裁切
			if ($target.data("vld-trim") != false) {
				value = $.trim(value);
			}
			// 考虑到有些情况会在文本框里填写输入提示
			// 如: <input value="用户名/手机号码/邮箱地址" />
			// 配置data-placeholder="用户名/手机号码/邮箱地址"
			if (value === $target.data("placeholder")) { value = ""; }
			return value;
		},
		// 1. data = false
		// 2. data = "key"
		// 3. data = { key:value }
		// 4. data = { "{key}:key, x:xvalue, y:yvalue }
		getTipsText: function(data) {
			var local = CONFIG.locals[CONFIG.lang];
			var keys = [this.rule.type], args = { label:this.label };
			if (typeof(data) == "string") { // 2
				keys.push(data);
			} else if ($.isPlainObject(data)) {
				if ("{key}" in data) { // 4
					var k = data["{key}"];
					if (k) { keys.push(k) }
					delete data["{key}"];
				} else { // 3
					for (var k in data) {
						// 只取一个
						if (k) { keys.push(k); break; }
					}
				}
				$.extend(args, data);
			}
			var key = keys.join("-");
			var msg = local[key];
			if (!msg) {
				msg = local[this.rule.type];
			}
			if (!msg) {
				msg = local["{default}"];
			}
			if (!msg) {
				msg = this.rule.type + " error";
			}
			return $.zhh.format(msg, args);
		},
		setTipsText: function(data, $tips) {
			var t = $tips || this.tips;
			// 是空的才设置默认提示
			var TIPS_IS_EMPTY = "TIPS_IS_EMPTY";
			if (t.data(TIPS_IS_EMPTY) === undefined) {
				var empty = t.children().length == 0 && $.trim(t.text()).length == 0;
				t.data(TIPS_IS_EMPTY, empty);
			}
			if (t.data(TIPS_IS_EMPTY) === true) {
				t.text(this.getTipsText(data));
			}
		}
	};

	var ajax = (function() {
		var exec = function(e) {
			var self = this;
			var passed = true;

			var LAST_VALUE = "vld-last-value",
				LAST_RESULT = "vld-last-result",
				FAIL_TEXT = "vld-fail-text";
			if(e.tips.data(LAST_VALUE) == e.value) { // 值没有变化
				if(e.tips.data(LAST_RESULT) == "1") {
					return true; // 之前已经校验通过了
				} else if(e.tips.data(LAST_RESULT) == "0") {
					e.tips.show(); return false; // 之前校验失败了
				}
			}

			var $fail = e.tips.find(".vld-fail"); // 校验不通过的样式
			var $error = e.tips.find(".vld-error"); // 出错的样式
			var $doing = e.tips.find(".vld-doing"); // 正在加载的样式

			// 记录当前的值, 以便下次校验前对比
			e.tips.data(LAST_VALUE, e.value);
			// 记录下原始的校验不通过的提示信息
			if (!e.tips.data(FAIL_TEXT)) {
				if ($fail.length == 0) {
					e.tips.data(FAIL_TEXT, e.tips.text());
				} else {
					e.tips.data(FAIL_TEXT, $fail.text());
				}
			}

			// 如果定义了正在加载的样式, 显示出来
			if ($doing.length > 0) {
				$fail.hide(); $doing.show(); $error.hide();
			}
			var fail = function(text) { // 校验不通过
				e.tips.data(LAST_RESULT, "0");
				if ($fail.length > 0) { // 定义了校验不通过的样式, 显示出来
					$fail.text(text || e.tips.data(FAIL_TEXT)).show();
					$doing.hide(); $error.hide();
				} else { // 没有定义校验不通过的样式, 直接显示文本
					e.tips.text(text || e.tips.data(FAIL_TEXT)).show();
				}
				passed = false;
			};
			var error = function(text) { // 报错
				// 报错则移除之前校验的结果, 以便提交时重新校验
				e.tips.removeData(LAST_RESULT);
				if ($error.length > 0) { // 定义了出错的样式, 显示出来
					$error.show(); $doing.hide(); $fail.hide();
				} else { // 没有定义出错的样式, 显示locals中配置的错误信息
					e.tips.text(e.getTipsText("error")).show();
					console && console.log(arguments);
				}
				passed = false;
			};
			var succ = function(data) {
				var json = CONFIG.ajax.parse.call(self, e, data);
				if(!json || json.passed === undefined) {
					// 没有获取到响应报文或者响应报文不符合格式
					error();
				} else if(json.passed) { // 校验通过
					e.tips.data(LAST_RESULT, "1").hide();
				} else { // 校验不通过
					fail(json.message);
				}
			};

			// 可以修改的请求参数
			var event = {
				url: e.tips.data("url") || CONFIG.ajax.url,
				type: e.tips.data("type"),
				dataType: e.tips.data("data-type") || "json",
				data: params(e)
			};
			// 不可修改的请求参数
			var def = { success:succ, error:error, async:false };
			// 发送请求前修改参数的机会
			CONFIG.ajax.prepare.call(self, $.extend(true, event, e));
			// 向服务器发送请求
			$.ajax($.extend(true, event, def));
			return passed;
		};
		var params = function(e) {
			// 如 data-params="type=1&name="
			var prefix = e.tips.data("params") || e.target.attr("name");
			// 删除最后的=号, 以防止=号重复
			prefix.replace(/=$/, "");
			return prefix + "=" + e.value;
		};
		// 准备向服务器发送请求前的处理
		var prepare = function(e) {
		};
		// 解析AJAX请求的响应报文
		var parse = function(e, json) {
			if (!json || typeof(json) != "string") { return json; }
			if(json == "true" || json == "1") { // 校验通过
				return { passed:true };
			} else if(json == "false" || json == "0") { // 校验不通过
				return { passed:false };
			} else if(json.length > (e.tips.data("max-length")||100)) {
				// 错误信息太长, 可能是后台报错, 打印了很多堆栈信息
				return { }; // 什么也不返回, 就是报错了
			} else { // 校验不通过, 返回了具体错误原因
				return { passed:false, message:json };
			}
		};
		return { exec:exec, prepare:prepare, parse:parse }
	})();

	var URL = new RegExp(
	"^("
		+ "(https?://)?" // 以http或https开头,也可以没有
		+ "("
		+     "[0-9]{1,3}([\\.][0-9]{1,3}){3}" // 4段的IP地址, 如192.168.88.99
		+ "|"
		+     "([^\\./\\?&@#]+[\\.])+" // 域名可以有任意多段, www.open.wo, baidu
		+     "[^\\./\\?&@#]{2,4}" // 后缀为2-4位, 如com, cn, info等
		+ ")"
		+ "(:[0-9]{2,})?" // 端口, 如:80, :443, :8080, 这一段可有可无
		+ "([/\\?#].*)?" // 网页地址或参数, 可为任意字符, 也可以没有
	+ ")?$", // 空字符串也可以通过校验
	"i"); // i:不区分大小写
	// 密码级别: 1.极弱|2.一般|3.较强|4.极强
	var pwdlevel = function(val) {
		if(val.length < CONFIG.pwd.minlength) { return 1; } // 小于6位, 弱爆了
		var count = 0,
			letter = /[a-zA-Z]+/, // 字母
			number = /[0-9]+/, // 数字
			other = /[^0-9a-zA-Z]+/; // 符号
		$.each([letter, number, other], function(i) {
			if (this.test(val)) { count ++; }
		});
		// 有3种类型且长度超过12位且符号超过4位为极强
		if (count >= 3 && val.length >= 12) {
			// 统计每一种类型的字符个数
			var list = [0, 0, 0];
			$.each(val.split(""), function(i, c) {
				$.each([letter, number, other], function(j) {
					if (this.test(c)) { list[j]++; }
				});
			});
			if (list[2] >= 4) { // 符号超过4位
				return 4;
			}
		}
		// 有3种为较强, 2种为一般, 1种为极弱
		return count >= 3 ? 3 : count >= 2 ? 2 : 1;
	};


	var CONFIG = $.fn.zhhvalidate.defaults = {
		locals: { },
		lang: "zh-CN",
		// 正则表达式, naming不可轻易修改, 其余正则表达式都可以修改
		// 如果修改naming, 会影响到vld-regexp的data-value
		// 和$.zhhvalidate.rules.put(name, fn)方法的name参数
		regexp: {
			// 命名规范:以字母数字下划线开头和结尾,中间可以有.和-但不能连续
			naming: /^[\w]+([\-\.][\w]+)*$/
		},
		ajax: {
			url:"ajaxvalidate.do",
			// 准备向服务器发送请求前的处理
			prepare:ajax.prepare,
			// 解析AJAX请求的响应报文
			parse:ajax.parse
		},
		pwd: {
			// 密码最小长度
			minlength: 6,
			// 检查密码级别的函数, 返回值: 1.极弱|2.一般|3.较强|4.极强
			level:pwdlevel,
			// 密码级别, 不可轻易修改, 如果需要修改, css文件也要跟着改
			levels: { 1:"weak", 2:"medium", 3:"strong", 4:"best" }
		},
		// 关键字, 不允许用作校验规则名称
		keyword: [ "form", "line", "target", "rule", "tips",
		    "label", "submit", "message", "passed",
			"doing", "succ", "fail", "error", "default",
			/pwd-(text|rate|weak|medium|strong|best)/ ],
		// 需要一直显示的规则, 提示规则, 而不是报错规则, 如密码强度提示
		visible: [ "pwd-tips" ],
		// 增加/获取需要绑定事件的元素, 参数为元素选择符
		// $.zhhvalidate.bind.input();  // 哪些元素需要在输入时校验
		// $.zhhvalidate.bind.blur();   // 哪些元素需要在失去焦点时校验
		// $.zhhvalidate.bind.click();  // 哪些元素需要在点击时校验
		// $.zhhvalidate.bind.submit(); // 哪些元素是提交按钮
		bind: bind,
		// 规则的获取/新增/修改/删除
		rules: rules
	};


	// 必填校验, 表示该字段必须输入, 如果为空则显示错误
	// 单选框复选框, 表示该字段必须选择至少一项, 如果未选择则显示错误
	CONFIG.rules.put("required", function(e) {
		if (e.target.is("input[type=radio], input[type=checkbox]")) {
			var checked = e.target.filter(":checked:visible");
			if (checked.length <= 0) { return "select"; }
		} else if (e.target.is("select")) {
			if (e.value.length <= 0) { return "select"; }
		} else if(e.value.length <= 0) {
			return "input";
		}
	});
	// 长度校验, 表示该字段必须符合长度要求, 符合为通过, 不符合则显示错误
	// data-min=最小值, data-max=最大值
	CONFIG.rules.put("length", function(e) {
		var len = e.value.length;
		if (len <= 0) { return true; }
		var min = parseInt(e.tips.data("min")),
			max = parseInt(e.tips.data("max"));
		if (!isNaN(min) && !isNaN(max)) { // min和max都有
			if(len < min || len > max) {
				return { "{key}":"min-max", min:min, max:max };
			}
		} else if (!isNaN(max)) { // 有max
			if (len > max) { return { max:max }; }
		} else if (!isNaN(min)) { // 有min
			if (len < min) { return { min:min }; }
		} else { // min和max都没有
			throw "data-min and data-max of vld-length is undefined.";
		}
	});
	// 数字校验, 表示该字段必须是数字, 符合为通过, 不符合则显示错误
	// data-min=最小值, data-max=最大值
	// data-sign=是否允许前面的+-号, 默认不允许
	// data-decimal=小数位数, 默认为0
	CONFIG.rules.put("number", function(e) {
		var len = e.value.length;
		if (len <= 0) { return true; }
		// 不是数字
		var trimed = $.trim(e.value);
		if (!/^[+-]?\d+(\.\d+)?$/.test(trimed)) {
			return "no"; // 不是数字
		}
		if (e.tips.data("sign") != true && /^[+-]/.test(trimed)) {
			return "sign"; // 不允许有符号, 但是有符号
		}
		var decimal = e.tips.data("decimal");
		if (!decimal) { // 0或undefined, 不允许小数
			if (trimed.indexOf(".") >= 0) {
				return "integer";
			}
		} else { // 允许小数
			var index = trimed.lastIndexOf(".");
			if (index >= 0 && trimed.length - index - 1 > decimal) {
				return { decimal:decimal };
			}
		}
		var value = parseFloat(trimed);
		var min = parseFloat(e.tips.data("min")),
			max = parseFloat(e.tips.data("max"));
		if (!isNaN(min) && !isNaN(max)) { // min和max都有
			if(value < min || value > max) {
				return { "{key}":"min-max", min:min, max:max };
			}
		} else if (!isNaN(max)) { // 有max
			if (value > max) { return { max:max }; }
		} else if (!isNaN(min)) { // 有min
			if (value < min) { return { min:min }; }
		}
	});
	// 正则表达式校验, 表示该字段必须匹配正则, 匹配为通过, 不匹配则显示错误
	// data-reverse=true表示反转判断结果, 如果匹配就提示错误
	// data-ignore-case=忽略大小写, true=忽略大小写, false=区分大小写(默认)
	CONFIG.rules.put("regexp", function(e) {
		if(e.value.length <= 0) { return true; }
		var text = e.tips.data("value");
		if (!text) { throw "data-value of vld-regexp is undefined."; }
		var regexp;
		// 符合命名规范的文本, 判定为指向CONFIG.regexp的名字
		// 如url=CONFIG.regexp.url, email=CONFIG.regexp.email
		// 命名规范:以字母数字下划线开头和结尾,中间可以有.和-但不能连续
		if (CONFIG.regexp.naming.test(text)) {
			regexp = CONFIG.regexp[text];
		}
		var match = true;
		if (regexp) { // 预定义的正则表达式
			// CONFIG.regexp.mobile = /^1[0-9]{10}$/;
			// CONFIG.regexp.illegal = { value:/[!@#$%^&*,.\/\\]/, reverse:true };
			if (regexp.constructor != RegExp) {
				match = regexp.reverse != true;
				regexp = regexp.value;
			}
			if(regexp.test(e.value) != match) {
				return text;
			}
		} else { // 根据配置生成正则表达式
			match = e.tips.data("reverse") != true;
			var ignore = e.tips.data("ignore-case") != true ? "" : "i";
			regexp = new RegExp(text, ignore);
			if(regexp.test(e.value) != match) {
				return false; // 无法做到通用提示, 只能提示格式有误
			}
		}
	});
	// 相等校验, 表示该字段与另一字段必须相等, 相等为通过, 不相等则显示错误
	// data-reverse=true表示反转判断结果, 如果相等就提示错误
	CONFIG.rules.put("compare", function(e) {
		if(e.value.length <= 0) { return true; }
		var match = e.tips.data("reverse") != true;
		var $form = e.tips.closest("form");
		var to = e.tips.data("compare-to");
		var $to = $form.find(to);
		var totext = e.findLabelText($to.closest(".vld-line"), $to);
		var tovalue = e.getValue($to);
		if((e.value == tovalue) != match) {
			if (match) {
				return { "{key}":"equal", target:totext };
			} else {
				return { "{key}":"notequal", target:totext };
			}
		}
	});
	// 密码强度提示, 这是一个提示性的规则, 不会返回false
	// 在CONFIG.visible配置了这个规则, 这样就不会隐藏
	// 如果要限制密码强度必须达到某一级别, 用vld-pwd-level规则
	CONFIG.rules.put("pwd-tips", function(e) {
		var TEXT = "d-vld-text",
			LEVEL = "d-vld-level";
		// 检查密码强度
		var level = CONFIG.pwd.level(e.value);
		if (level == e.tips.data(LEVEL)) { // 跟之前的级别一样
			return true;
		}
		// 记录下最后一次的级别
		e.tips.data(LEVEL, level);

		var levels = CONFIG.pwd.levels;
		for (var key in levels) {
			e.tips.removeClass("vld-pwd-" + levels[key]);
		}
		// 记录下原始的显示文本
		var $text = e.tips.find(".vld-pwd-text"),
			text = e.tips.data(TEXT);
		!text && e.tips.data(TEXT, (text = $text.text()));
		if(e.value.length <= 0) {
			$text.text(text); // 还原
			return true;
		}
		// 设置密码强度
		e.tips.addClass("vld-pwd-" + levels[level]);
		$text.text(text + e.tips.data(levels[level]));
	});
	// 密码强度校验, 必须密码达到指定级别, 否则提示错误
	CONFIG.rules.put("pwd-level", function(e) {
		if(e.value.length <= 0) { return true; }
		var level = e.tips.data("level") || 2;
		// 如果不是数字, 转换为数字
		if (/[^0-9]/.test(level)) {
			var levels = CONFIG.pwd.levels;
			var text = level; level = 0;
			for (var key in levels) {
				if (levels[key] == text) { level = key; }
			}
		}
		// 如果小于1或大于4, 默认为2
		if (level < 1 || level > 4) { level = 2; }
		if(CONFIG.pwd.level(e.value) < level * 1) {
			return CONFIG.pwd.levels[level];
		}
	});
	// 通过ajax向服务器请求校验
	CONFIG.rules.put("ajax", function(e) {
		if(e.value.length <= 0) { return true; }
		return ajax.exec(e);
	});

	// 预定义正则表达式
	CONFIG.regexp.url = URL;
	CONFIG.regexp.ascii = /^[\x00-\x7F]*$/; // 不支持中文
	CONFIG.regexp.mobile = /^1[0-9]{10}$/;
	CONFIG.regexp.email = /^[^\s@]+@([^\s@\.]+\.)+[^\s@\.]{2,4}$/;
	// 不允许输入非法字符
	// 正则表达式校验, 表示该字段必须匹配正则, 匹配为通过, 不匹配则显示错误
	// reverse:true表示反转判断结果, 如果匹配就提示错误
	CONFIG.regexp["illegal-char"] = { value:/[!@#$%^&*,.\/\\]/, reverse:true };

	CONFIG.locals["zh-CN"] = {
		"{default}": "{label}输入有误",

		// 必填校验, 表示该字段必须输入, 如果为空则显示错误
		// 单选框复选框, 表示该字段必须选择至少一项, 如果未选择则显示错误
		"required-input":"{label}不能为空",
		"required-select":"请选择一项",

		// min=最小值, max=最大值
		"length-min":"长度不能小于{min}位",
		"length-max":"长度不能大于{max}位",
		"length-min-max":"长度必须在{min}至{max}位之间",

		// 数字校验, 表示该字段必须是数字, 符合为通过, 不符合则显示错误
		// min=最小值, max=最大值
		// sign=是否允许前面的+-号, 默认不允许
		// decimal=小数位数, 默认为0
		"number-no":"请输入数字",
		"number-integer":"请输入整数",
		"number-decimal":"小数位数不能大于{decimal}位",
		"number-sign":"不允许输入+-号",
		"number-min":"请输入不小于{min}的数字",
		"number-max":"请输入不大于{max}的数字",
		"number-min-max":"请输入{min}至{max}之间的数字",

		// 相等校验, 表示该字段与另一字段必须相等, 相等为通过, 不相等则显示错误
		// data-reverse=true表示反转判断结果, 如果相等就提示错误
		"compare-equal":"{label}与{target}不符",
		"compare-notequal":"{label}不能与{target}相同",

		// 密码级别  { 1:"weak", 2:"medium", 3:"strong", 4:"best" }
		"pwd-level-weak":  "{label}必须包含字母/数字/特殊字符的任意一种",
		"pwd-level-medium":"{label}必须包含字母/数字/特殊字符的任意两种",
		"pwd-level-strong":"{label}必须包含字母/数字/特殊字符",
		"pwd-level-best":  "{label}必须包含字母/数字/特殊字符且至少12位且特殊字符超过4位",

		// AJAX的默认错误提示
		"ajax-error": "{label}校验出错",

		// 正则表达式默认错误提示
		"regexp": "{label}格式有误",
		"regexp-url": "网址格式有误",
		"regexp-mobile": "手机号码格式有误",
		"regexp-email": "邮箱地址格式有误",
		"regexp-ascii": "{label}只支持英文字符",
		"regexp-illegal-char": "{label}不支持!@#$%^&*,./\等特殊字符"
	};

})(jQuery);
jQuery(function($) {
	"use strict";
	$("form.vld-form").zhhvalidate("init");
});
